"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BbPromise = require("bluebird");
const IORedis = require("ioredis");
class RedisDriver {
    constructor(serverUrl, options = {}) {
        this.serverUrl = serverUrl;
        this.options = options;
        this.isCluster = !!options.enableClusterMode;
        if (this.isCluster) {
            const DEFAULT_OPTIONS = {
                clusterRetryStrategy(times) {
                    return 100; // retry after 100ms
                },
            };
            // if server url is master, node discovery is automatically performed
            this.client = new IORedis.Cluster([serverUrl], Object.assign({}, DEFAULT_OPTIONS, (options.ioredis || {})));
        }
        else {
            const DEFAULT_OPTIONS = {
                retryStrategy(times) {
                    return 100; // retry after 100ms
                },
            };
            this.client = new IORedis(serverUrl, Object.assign({}, DEFAULT_OPTIONS, (options.ioredis || {})));
        }
    }
    async touch(key, lifetime) {
        // @see https://redis.io/commands/expire#return-value
        // @type Integer reply
        //
        // 1 if the timeout was set.
        // 0 if key does not exist.
        const reply = await this.client.expire(key, lifetime);
        return reply === 1;
    }
    async get(key) {
        const response = await this.client.get(key);
        try {
            return JSON.parse(response);
        }
        catch (e) {
            return response;
        }
    }
    async getMulti(keys) {
        if (keys.length === 0) {
            return {};
        }
        // In cluster mode, MGET (multiple get) command requires all keys must be same key slot
        // if client does not handle this, redis will give "CROSSSLOT Keys in request don't hash to the same slot" Error
        if (this.isCluster) {
            // this is temporary fix for CROSSSLOT issue.
            // we should consider using cluster-key-slot module to optimize requests.
            // @todo optimize getMulti logic in Cluster Mode
            return await BbPromise.reduce(keys, async (hash, key) => {
                hash[key] = await this.get(key);
                return hash;
            }, {});
        }
        else {
            const response = await this.client.mget(...keys);
            return keys.reduce((hash, key, index) => {
                const val = response[index];
                if (val !== null) {
                    try {
                        hash[key] = JSON.parse(val);
                    }
                    catch (e) {
                        hash[key] = val;
                    }
                    return hash;
                }
                hash[key] = undefined;
                return hash;
            }, {});
        }
    }
    async set(key, value, lifetime) {
        const serialized = JSON.stringify(value);
        // @see https://redis.io/commands/setex#return-value
        // @type Simple string Reply
        let reply;
        if (!lifetime) {
            reply = await this.client.set(key, serialized);
        }
        else {
            reply = await this.client.setex(key, lifetime, serialized);
        }
        if (reply !== "OK") {
            throw new Error(`RedisDriver failed to set: '${key} - ${value}'`);
        }
    }
    async del(key) {
        // @see https://redis.io/commands/del#return-value
        // @type Integer reply: The number of keys that were removed.
        const reply = await this.client.del(key);
        return reply > 0;
    }
    async flush() {
        // @see https://redis.io/commands/flushdb
        // @type Simple string reply
        //
        // @note starting redis 4, redis support **ASYNC** flush mode.
        // since redis blocks everything during flush process,
        // flushing database which has huge number of keys can be cause redis server outage.
        // so it is great improvement, but currently AWS ElastiCache does not support redis 4.
        // @note this command does not flush entire redis database,
        // so this command **ONLY** flushes current active database
        const reply = await this.client.flushdb();
        if (reply !== "OK") {
            throw new Error(`RedisDriver failed to flush: '${reply}'`);
        }
    }
    async end() {
        await this.client.quit(); // quit method should disconnect connection cleanly
    }
}
exports.RedisDriver = RedisDriver;
//# sourceMappingURL=redis.js.map