"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("crypto");
function isNotUndefinedOrNull(x) {
    return x !== undefined && x !== null;
}
class MemcachedFetcher {
    constructor(driver, options = {}) {
        this.driver = driver;
        const keyHashing = options.keyHashing !== undefined ? options.keyHashing : true;
        if (keyHashing) {
            this.keyHasher = (key) => {
                return crypto
                    .createHash("md5")
                    .update(key)
                    .digest("hex");
            };
        }
        else {
            this.keyHasher = (key) => key;
        }
    }
    async fetch(key, lifetime, fetcher) {
        const hasheKey = this.keyHasher(key);
        let value = await this.driver.get(hasheKey);
        if (!isNotUndefinedOrNull(value)) {
            value = await fetcher();
            await this.driver.set(hasheKey, value, lifetime);
            return value;
        }
        return value;
    }
    async del(key) {
        const hashedKey = this.keyHasher(key);
        return await this.driver.del(hashedKey);
    }
    async multiFetch(args, key, lifetime, fetcher) {
        if (args.length === 0) {
            return [];
        }
        const { namespace, argToKey } = (() => {
            if (typeof key === "string") {
                return { namespace: key, argToKey: (arg) => String(arg) };
            }
            else {
                return { namespace: key[0], argToKey: key[1] };
            }
        })();
        const argsToKeyMap = new Map(args.map((arg) => {
            const hashedKey = this.keyHasher(`${namespace}:${argToKey(arg).toString()}`);
            return [arg, hashedKey];
        }));
        const cached = (await this.driver.getMulti(Array.from(argsToKeyMap.values())));
        const missingArgs = args.filter((arg) => !isNotUndefinedOrNull(cached[argsToKeyMap.get(arg)]));
        const fetchedArray = missingArgs.length > 0 ? await fetcher(missingArgs) : [];
        if (fetchedArray.length !== missingArgs.length) {
            throw new Error("Fetcher must return same length of result with Args.length");
        }
        const fetched = new Map(missingArgs.map((arg, index) => [arg, fetchedArray[index]]));
        await Promise.all(Array.from(fetched).map(async ([arg, result]) => {
            await this.driver.set(argsToKeyMap.get(arg), result, lifetime);
        }));
        return args.map((arg) => {
            const cacheKey = argsToKeyMap.get(arg);
            const value = cached[cacheKey];
            if (isNotUndefinedOrNull(value)) {
                return value;
            }
            else {
                return fetched.get(arg);
            }
        });
    }
    async multiFetchDelete(args, key) {
        const { namespace, argToKey } = (() => {
            if (typeof (key) === "string") {
                return { namespace: key, argToKey: (arg) => String(arg) };
            }
            else {
                return { namespace: key[0], argToKey: key[1] };
            }
        })();
        await Promise.all(args.map(async (arg) => {
            const hashedKey = this.keyHasher(`${namespace}:${argToKey(arg).toString()}`);
            await this.driver.del(hashedKey);
        }));
    }
}
exports.MemcachedFetcher = MemcachedFetcher;
//# sourceMappingURL=fetcher.js.map